@@ -9,6 +9,10 @@ fn main() {
     let target = env::var("TARGET").unwrap();
     let windows = target.contains("windows");
 
+    if cfg!(feature = "mesalink") {
+        println!("cargo:warning=MesaLink support has been removed as of curl 7.82.0, will use default TLS backend instead.");
+    }
+
     // This feature trumps all others, and is largely set by rustbuild to force
     // usage of the system library to ensure that we're always building an
     // ABI-compatible Cargo.
@@ -72,12 +76,14 @@ fn main() {
         "curlver.h",
         "easy.h",
         "options.h",
+        "header.h",
         "mprintf.h",
         "multi.h",
         "stdcheaders.h",
         "system.h",
         "urlapi.h",
         "typecheck-gcc.h",
+        "websockets.h",
     ]
     .iter()
     {
@@ -116,7 +122,6 @@ fn main() {
         .define("CURL_DISABLE_IMAP", None)
         .define("CURL_DISABLE_LDAP", None)
         .define("CURL_DISABLE_LDAPS", None)
-        .define("CURL_DISABLE_NTLM", None)
         .define("CURL_DISABLE_POP3", None)
         .define("CURL_DISABLE_RTSP", None)
         .define("CURL_DISABLE_SMB", None)
@@ -141,30 +146,28 @@ fn main() {
         .file("curl/lib/content_encoding.c")
         .file("curl/lib/cookie.c")
         .file("curl/lib/curl_addrinfo.c")
-        .file("curl/lib/curl_ctype.c")
         .file("curl/lib/curl_get_line.c")
         .file("curl/lib/curl_memrchr.c")
         .file("curl/lib/curl_range.c")
         .file("curl/lib/curl_threads.c")
-        .file("curl/lib/dotdot.c")
         .file("curl/lib/doh.c")
         .file("curl/lib/dynbuf.c")
         .file("curl/lib/easy.c")
         .file("curl/lib/escape.c")
         .file("curl/lib/file.c")
         .file("curl/lib/fileinfo.c")
+        .file("curl/lib/fopen.c")
         .file("curl/lib/formdata.c")
         .file("curl/lib/getenv.c")
         .file("curl/lib/getinfo.c")
         .file("curl/lib/hash.c")
+        .file("curl/lib/headers.c")
         .file("curl/lib/hmac.c")
         .file("curl/lib/hostasyn.c")
-        .file("curl/lib/hostcheck.c")
         .file("curl/lib/hostip.c")
         .file("curl/lib/hostip6.c")
         .file("curl/lib/hsts.c")
         .file("curl/lib/http.c")
-        .file("curl/lib/http2.c")
         .file("curl/lib/http_aws_sigv4.c")
         .file("curl/lib/http_chunks.c")
         .file("curl/lib/http_digest.c")
@@ -180,6 +183,7 @@ fn main() {
         .file("curl/lib/multi.c")
         .file("curl/lib/netrc.c")
         .file("curl/lib/nonblock.c")
+        .file("curl/lib/noproxy.c")
         .file("curl/lib/parsedate.c")
         .file("curl/lib/progress.c")
         .file("curl/lib/rand.c")
@@ -206,15 +210,31 @@ fn main() {
         .file("curl/lib/version.c")
         .file("curl/lib/vauth/digest.c")
         .file("curl/lib/vauth/vauth.c")
+        .file("curl/lib/vtls/hostcheck.c")
         .file("curl/lib/vtls/keylog.c")
         .file("curl/lib/vtls/vtls.c")
         .file("curl/lib/warnless.c")
         .file("curl/lib/wildcard.c")
+        .file("curl/lib/timediff.c")
         .define("HAVE_GETADDRINFO", None)
         .define("HAVE_GETPEERNAME", None)
         .define("HAVE_GETSOCKNAME", None)
         .warnings(false);
 
+    if cfg!(feature = "ntlm") {
+        cfg.file("curl/lib/curl_des.c")
+            .file("curl/lib/curl_endian.c")
+            .file("curl/lib/curl_gethostname.c")
+            .file("curl/lib/curl_ntlm_core.c")
+            .file("curl/lib/curl_ntlm_wb.c")
+            .file("curl/lib/http_ntlm.c")
+            .file("curl/lib/md4.c")
+            .file("curl/lib/vauth/ntlm.c")
+            .file("curl/lib/vauth/ntlm_sspi.c");
+    } else {
+        cfg.define("CURL_DISABLE_NTLM", None);
+    }
+
     if cfg!(feature = "protocol-ftp") {
         cfg.file("curl/lib/curl_fnmatch.c")
             .file("curl/lib/ftp.c")
@@ -226,7 +246,9 @@ fn main() {
 
     if cfg!(feature = "http2") {
         cfg.define("USE_NGHTTP2", None)
-            .define("NGHTTP2_STATICLIB", None);
+            .define("NGHTTP2_STATICLIB", None)
+            .file("curl/lib/h2h3.c")
+            .file("curl/lib/http2.c");
 
         println!("cargo:rustc-cfg=link_libnghttp2");
         if let Some(path) = env::var_os("DEP_NGHTTP2_ROOT") {
@@ -248,18 +270,29 @@ fn main() {
 
     // Configure TLS backend. Since Cargo does not support mutually exclusive
     // features, make sure we only compile one vtls.
-    if cfg!(feature = "mesalink") {
-        cfg.define("USE_MESALINK", None)
-            .file("curl/lib/vtls/mesalink.c");
-
-        if let Some(path) = env::var_os("DEP_MESALINK_INCLUDE") {
-            cfg.include(path);
-        }
-
+    if cfg!(feature = "rustls") {
+        cfg.define("USE_RUSTLS", None)
+            .file("curl/lib/vtls/rustls.c")
+            .include(env::var_os("DEP_RUSTLS_FFI_INCLUDE").unwrap());
+    } else if cfg!(feature = "windows-static-ssl") {
         if windows {
-            cfg.define("HAVE_WINDOWS", None);
+            cfg.define("USE_OPENSSL", None)
+                .file("curl/lib/vtls/openssl.c");
+            // We need both openssl and zlib
+            // Those can be installed with
+            // ```shell
+            // git clone https://github.com/microsoft/vcpkg
+            // cd vcpkg
+            // ./bootstrap-vcpkg.bat -disableMetrics
+            // ./vcpkg.exe integrate install
+            // ./vcpkg.exe install openssl:x64-windows-static-md
+            // ```
+            #[cfg(target_env = "msvc")]
+            vcpkg::Config::new().find_package("openssl").ok();
+            #[cfg(target_env = "msvc")]
+            vcpkg::Config::new().find_package("zlib").ok();
         } else {
-            cfg.define("HAVE_UNIX", None);
+            panic!("Not available on non windows platform")
         }
     } else if cfg!(feature = "ssl") {
         if windows {
@@ -268,17 +301,17 @@ fn main() {
             cfg.define("USE_WINDOWS_SSPI", None)
                 .define("USE_SCHANNEL", None)
                 .file("curl/lib/http_negotiate.c")
-                .file("curl/lib/x509asn1.c")
                 .file("curl/lib/curl_sspi.c")
                 .file("curl/lib/socks_sspi.c")
                 .file("curl/lib/vauth/spnego_sspi.c")
                 .file("curl/lib/vauth/vauth.c")
                 .file("curl/lib/vtls/schannel.c")
-                .file("curl/lib/vtls/schannel_verify.c");
+                .file("curl/lib/vtls/schannel_verify.c")
+                .file("curl/lib/vtls/x509asn1.c");
         } else if target.contains("-apple-") {
             cfg.define("USE_SECTRANSP", None)
-                .file("curl/lib/x509asn1.c")
-                .file("curl/lib/vtls/sectransp.c");
+                .file("curl/lib/vtls/sectransp.c")
+                .file("curl/lib/vtls/x509asn1.c");
             if xcode_major_version().map_or(true, |v| v >= 9) {
                 // On earlier Xcode versions (<9), defining HAVE_BUILTIN_AVAILABLE
                 // would cause __bultin_available() to fail to compile due to
@@ -497,14 +530,17 @@ fn try_pkg_config() -> bool {
 }
 
 fn xcode_major_version() -> Option<u8> {
-    let output = Command::new("xcodebuild").arg("-version").output().ok()?;
-    if output.status.success() {
-        let stdout = String::from_utf8_lossy(&output.stdout);
-        println!("xcode version: {}", stdout);
-        let mut words = stdout.split_whitespace();
-        if words.next()? == "Xcode" {
-            let version = words.next()?;
-            return version[..version.find('.')?].parse().ok();
+    let status = Command::new("xcode-select").arg("-p").status().ok()?;
+    if status.success() {
+        let output = Command::new("xcodebuild").arg("-version").output().ok()?;
+        if output.status.success() {
+            let stdout = String::from_utf8_lossy(&output.stdout);
+            println!("xcode version: {}", stdout);
+            let mut words = stdout.split_whitespace();
+            if words.next()? == "Xcode" {
+                let version = words.next()?;
+                return version[..version.find('.')?].parse().ok();
+            }
         }
     }
     println!("unable to determine Xcode version, assuming >= 9");
@@ -537,7 +573,9 @@ fn curl_config_reports_http2() -> bool {
 }
 
 fn macos_link_search_path() -> Option<String> {
-    let output = Command::new("clang")
+    let output = cc::Build::new()
+        .get_compiler()
+        .to_command()
         .arg("--print-search-dirs")
         .output()
         .ok()?;
@@ -551,8 +589,10 @@ fn macos_link_search_path() -> Option<String> {
     let stdout = String::from_utf8_lossy(&output.stdout);
     for line in stdout.lines() {
         if line.contains("libraries: =") {
-            let path = line.split('=').skip(1).next()?;
-            return Some(format!("{}/lib/darwin", path));
+            let path = line.split('=').nth(1)?;
+            if !path.is_empty() {
+                return Some(format!("{}/lib/darwin", path));
+            }
         }
     }
 
